C ========================================================
C  ATPDesigner - Design and Simulation of Power Networks
C  Version 2.00.42 - 27.12.2008
C  Copyright  2005-09
C  29.12.2008 , 19:27:33
C ========================================================
C ATPDESIGNER START BLOCK MODELS
MODELS -- Start of the MODELS Section

  -- ATPDESIGNER START MODEL CLARKE
  -- Clarke - Transformation (ABC -> 0ab)
  MODEL CLARKE

    -- Input Interface at MODEL level : MODELS -> MODEL source code
    INPUT API  -- Current Phase IA
    INPUT BPI  -- Current Phase IB
    INPUT CPI  -- Current Phase IC
    INPUT APV  -- Phase-to-Ground Voltage VAG
    INPUT BPV  -- Phase-to-Ground Voltage VBG
    INPUT CPV  -- Phase-to-Ground Voltage VCG

    CONST PowerNetFreq {val: 50}  -- POWER FREQUENCY [Hz]
    CONST PowerNetCycle {val: 0.02}  -- Cycle [s]
    CONST SamplingFreq {val: 1000}  -- Sampling Frequency [Hz]
    CONST SamplingDT {val: 0.001}  -- Sampling Rate [s]
    CONST SamplesPerCycle {val: 20}  -- Samples per Cycle

    DATA MeasLocation {dflt: -1.0} -- ID of the measuring location

    CONST RadToDeg {val: 180.0/pi}

    -- Declaration of all variables
    VAR

      -- Sampled 0ab-voltages of the last cycle
      V0Buffer[0..(SamplesPerCycle - 1)]
      VaBuffer[0..(SamplesPerCycle - 1)]
      VbBuffer[0..(SamplesPerCycle - 1)]

      -- Sampled 0ab-currents of the last cycle
      I0Buffer[0..(SamplesPerCycle - 1)]
      IaBuffer[0..(SamplesPerCycle - 1)]
      IbBuffer[0..(SamplesPerCycle - 1)]

      -- Filter coefficients (Discrete Fourier Analysis)
      CosKoeff[0..(SamplesPerCycle - 1)]
      SinKoeff[0..(SamplesPerCycle - 1)]

      -- Re=Real part, Im=Imaginary part
      -- Amount in V or A, Phase in Degree
      V0Re, V0Im
      VaRe, VaIm
      VbRe, VbIm
      I0Re, I0Im
      IaRe, IaIm
      IbRe, IbIm
      V0Amount, V0Phase
      VaAmount, VaPhase
      VbAmount, VbPhase
      I0Amount, I0Phase
      IaAmount, IaPhase
      IbAmount, IbPhase
      Omega -- Angular Frequency in Hz

      -- Sampled values 0ab-system
      -- V=Voltage, I=Current
      V0
      Va
      Vb
      I0
      Ia
      Ib

      -- Coefficients of the Clarke - transformation ABC -> 0ab
      -- for phase-to-ground voltages
      -- |V0|   |TV1[1] TV1[2] TV1[3]|   |VAG|
      -- |Va| = |TV2[1] TV2[2] TV2[3]| x |VBG|
      -- |Vb|   |TV3[1] TV3[2] TV3[3]|   |VCG|
      TV1[1..3]  -- Line 1
      TV2[1..3]  -- Line 2
      TV3[1..3]  -- Line 3

      -- Coefficients of the Clarke - transformation ABC -> 0ab
      -- for phase currents
      -- |I0|   |TI1[1] TI1[2] TI1[3]|   |IAG|
      -- |Ia| = |TI2[1] TI2[2] TI2[3]| x |IBG|
      -- |Ib|   |TI3[1] TI3[2] TI3[3]|   |ICG|
      TI1[1..3]  -- Line 1
      TI2[1..3]  -- Line 2
      TI3[1..3]  -- Line 3

    -- One-time initialization of variables, etc. at simulation start
    INIT

      -- Coefficients of the Clarke - transformation ABC -> 0ab
      -- for transformation of phase-to-ground voltages
      TV1[1] := 1/3
      TV1[2] := 1/3
      TV1[3] := 1/3
      TV2[1] := 2/3
      TV2[2] := -1/3
      TV2[3] := -1/3
      TV3[1] := 0
      TV3[2] := sqrt(3)/3
      TV3[3] := -1*sqrt(3)/3

      -- Coefficients of the Clarke - transformation ABC -> 0ab
      -- for transformation of phase currents
      TI1[1] := 1/3
      TI1[2] := 1/3
      TI1[3] := 1/3
      TI2[1] := 2/3
      TI2[2] := -1/3
      TI2[3] := -1/3
      TI3[1] := 0
      TI3[2] := sqrt(3)/3
      TI3[3] := -1*sqrt(3)/3

      V0 := 0.0
      Va := 0.0
      Vb := 0.0
      I0 := 0.0
      Ia := 0.0
      Ib := 0.0

      for i := 0 to (SamplesPerCycle - 1) do
        V0Buffer[i] := 0.0
        VaBuffer[i] := 0.0
        VbBuffer[i] := 0.0
        I0Buffer[i] := 0.0
        IaBuffer[i] := 0.0
        IbBuffer[i] := 0.0
      endfor

      V0Re := 0.0
      V0Im := 0.0
      VaRe := 0.0
      VaIm := 0.0
      VbRe := 0.0
      VbIm := 0.0
      I0Re := 0.0
      I0Im := 0.0
      IaRe := 0.0
      IaIm := 0.0
      IbRe := 0.0
      IbIm := 0.0
      V0Amount := 0.0
      VaPhase := 0.0
      VbAmount := 0.0
      I0Amount := 0.0
      IaPhase := 0.0
      IbAmount := 0.0
      V0Phase := 0.0
      VaAmount := 0.0
      VbPhase := 0.0
      I0Phase := 0.0
      IaAmount := 0.0
      IbPhase := 0.0
      Omega := 2.0 * pi * PowerNetFreq

      -- Calculation of DFT Filter Coefficients
      for i := 0 to (SamplesPerCycle - 1) do
        CosKoeff[i] := cos(Omega * SamplingDT * i)
        SinKoeff[i] := sin(Omega * SamplingDT * i)
      endfor

    ENDINIT

    -- Execution at each simulation time step
    EXEC

      -- Shifting the sampled values one time step to the past
      for i := 0 to (SamplesPerCycle - 2) do
        V0Buffer[i] := V0Buffer[i+1]
        VaBuffer[i] := VaBuffer[i+1]
        VbBuffer[i] := VbBuffer[i+1]
        I0Buffer[i] := I0Buffer[i+1]
        IaBuffer[i] := IaBuffer[i+1]
        IbBuffer[i] := IbBuffer[i+1]
      endfor

      -- Transformation: ABC -> 0ab
      V0 := (TV1[1] * APV) + (TV1[2] * BPV) + (TV1[3] * CPV)
      Va := (TV2[1] * APV) + (TV2[2] * BPV) + (TV2[3] * CPV)
      Vb := (TV3[1] * APV) + (TV3[2] * BPV) + (TV3[3] * CPV)
      I0 := (TI1[1] * API) + (TI1[2] * BPI) + (TI1[3] * CPI)
      Ia := (TI2[1] * API) + (TI2[2] * BPI) + (TI2[3] * CPI)
      Ib := (TI3[1] * API) + (TI3[2] * BPI) + (TI3[3] * CPI)

      -- Insert the new sampled value
      V0Buffer[SamplesPerCycle - 1] := V0
      VaBuffer[SamplesPerCycle - 1] := Va
      VbBuffer[SamplesPerCycle - 1] := Vb
      I0Buffer[SamplesPerCycle - 1] := I0
      IaBuffer[SamplesPerCycle - 1] := Ia
      IbBuffer[SamplesPerCycle - 1] := Ib

      -- Reset variables
      V0Re := 0.0
      V0Im := 0.0
      VaRe := 0.0
      VaIm := 0.0
      VbRe := 0.0
      VbIm := 0.0
      I0Re := 0.0
      I0Im := 0.0
      IaRe := 0.0
      IaIm := 0.0
      IbRe := 0.0
      IbIm := 0.0
      V0Amount := 0.0
      VaPhase := 0.0
      VbAmount := 0.0
      I0Amount := 0.0
      IaPhase := 0.0
      IbAmount := 0.0
      V0Phase := 0.0
      VaAmount := 0.0
      VbPhase := 0.0
      I0Phase := 0.0
      IaAmount := 0.0
      IbPhase := 0.0

      -- Processing the DFT for the last cycle
      for i := 0 to (SamplesPerCycle - 1) do
        V0Re := V0Re + V0Buffer[i] * CosKoeff[i]
        V0Im := V0Im + V0Buffer[i] * SinKoeff[i]
        VaRe := VaRe + VaBuffer[i] * CosKoeff[i]
        VaIm := VaIm + VaBuffer[i] * SinKoeff[i]
        VbRe := VbRe + VbBuffer[i] * CosKoeff[i]
        VbIm := VbIm + VbBuffer[i] * SinKoeff[i]
        I0Re := I0Re + I0Buffer[i] * CosKoeff[i]
        I0Im := I0Im + I0Buffer[i] * SinKoeff[i]
        IaRe := IaRe + IaBuffer[i] * CosKoeff[i]
        IaIm := IaIm + IaBuffer[i] * SinKoeff[i]
        IbRe := IbRe + IbBuffer[i] * CosKoeff[i]
        IbIm := IbIm + IbBuffer[i] * SinKoeff[i]
      endfor

      V0Re := V0Re * 2.0 / SamplesPerCycle
      V0Im := V0Im * 2.0 / SamplesPerCycle
      VaRe := VaRe * 2.0 / SamplesPerCycle
      VaIm := VaIm * 2.0 / SamplesPerCycle
      VbRe := VbRe * 2.0 / SamplesPerCycle
      VbIm := VbIm * 2.0 / SamplesPerCycle
      I0Re := I0Re * 2.0 / SamplesPerCycle
      I0Im := I0Im * 2.0 / SamplesPerCycle
      IaRe := IaRe * 2.0 / SamplesPerCycle
      IaIm := IaIm * 2.0 / SamplesPerCycle
      IbRe := IbRe * 2.0 / SamplesPerCycle
      IbIm := IbIm * 2.0 / SamplesPerCycle

      -- r.m.s value of voltages and currents of the fundamental frequency
      V0Amount := sqrt((V0Re * V0Re) + (V0Im * V0Im)) / sqrt(2)
      VaAmount := sqrt((VaRe * VaRe) + (VaIm * VaIm)) / sqrt(2)
      VbAmount := sqrt((VbRe * VbRe) + (VbIm * VbIm)) / sqrt(2)
      I0Amount := sqrt((I0Re * I0Re) + (I0Im * I0Im)) / sqrt(2)
      IaAmount := sqrt((IaRe * IaRe) + (IaIm * IaIm)) / sqrt(2)
      IbAmount := sqrt((IbRe * IbRe) + (IbIm * IbIm)) / sqrt(2)

      -- Absolute phase angle of the fundamental frequency
      if (V0Im <> 0.0) and (V0Re <> 0.0) then
        V0Phase := atan2(V0Im, V0Re) * RadToDeg
      endif

      if (VaIm <> 0.0) and (VaRe <> 0.0) then
        VaPhase := atan2(VaIm, VaRe) * RadToDeg
      endif

      if (VbIm <> 0.0) and (VbRe <> 0.0) then
        VbPhase := atan2(VbIm, VbRe) * RadToDeg
      endif

      if (I0Im <> 0.0) and (I0Re <> 0.0) then
        I0Phase := atan2(I0Im, I0Re) * RadToDeg
      endif

      if (IaIm <> 0.0) and (IaRe <> 0.0) then
        IaPhase := atan2(IaIm, IaRe) * RadToDeg
      endif

      if (IbIm <> 0.0) and (IbRe <> 0.0) then
        IbPhase := atan2(IbIm, IbRe) * RadToDeg
      endif

    ENDEXEC

  ENDMODEL
  -- End of the MODEL implementation
  -- ATPDESIGNER END MODEL CLARKE

  -- ATPDESIGNER START BLOCK USE CLARKE AS CLARKE001
  -- Input Interface at MODELS level: Power Network -> MODELS
  -- Interface from Measuring Probe Prb 001
  INPUT APV001 {TACS(APV001)}
  INPUT BPV001 {TACS(BPV001)}
  INPUT CPV001 {TACS(CPV001)}
  INPUT API001 {TACS(API001)}
  INPUT BPI001 {TACS(BPI001)}
  INPUT CPI001 {TACS(CPI001)}

  -- ATPDESIGNER START USE CLARKE001
  -- Start of the MODEL instance (task)
  USE CLARKE AS CLARKE001

    -- Phase Current A, B, C
    INPUT API := API001
    INPUT BPI := BPI001
    INPUT CPI := CPI001

    INPUT APV := APV001
    INPUT BPV := BPV001
    INPUT CPV := CPV001

  ENDUSE
  -- End of the MODEL instance (task)
  -- ATPDESIGNER END USE CLARKE001

  -- ATPDESIGNER START RECORD 
  -- Recording Signals to .PL4-File
  RECORD
    CLARKE001.V0 AS V0001
    CLARKE001.Va AS VA001
    CLARKE001.Vb AS VB001
    CLARKE001.I0 AS I0001
    CLARKE001.Ia AS IA001
    CLARKE001.Ib AS IB001
    CLARKE001.V0Amount AS AV0001
    CLARKE001.VaAmount AS AVA001
    CLARKE001.VbAmount AS AVB001
    CLARKE001.I0Amount AS AI0001
    CLARKE001.IaAmount AS AIA001
    CLARKE001.IbAmount AS AIB001
    CLARKE001.V0Phase AS PV0001
    CLARKE001.VaPhase AS PVA001
    CLARKE001.VbPhase AS PVB001
    CLARKE001.I0Phase AS PI0001
    CLARKE001.IaPhase AS PIA001
    CLARKE001.IbPhase AS PIB001
  -- ATPDESIGNER END RECORD 
  -- ATPDESIGNER END BLOCK USE CLARKE AS CLARKE001

ENDMODELS -- End of the MODELS Section
C ATPDESIGNER END BLOCK MODELS
